# Process the incoming configuration files

if [ ! -f etc/ax25/soundmodem.conf ]
then
   mv install/conf/etc/ax25/soundmodem.conf etc/ax25/soundmodem.conf
else
   if [ "`md5sum install/conf/etc/ax25/soundmodem.conf | cut -d ' ' -f 1 2>/dev/null`" = "`md5sum etc/ax25/soundmodem.conf | cut -d ' ' -f 1 2>/dev/null`" ]
   then
      mv install/conf/etc/ax25/soundmodem.conf etc/ax25/soundmodem.conf
   else
      rm install/conf/etc/ax25/soundmodem.conf
   fi
fi

# Make sure the incoming conf directory is gone
rm -rf install/conf
